@@ -44,6 +44,7 @@
 #include "MagickWand/studio.h"
 #include "MagickWand/MagickWand.h"
 #include "MagickWand/mogrify-private.h"
+#include "MagickCore/image-private.h"
 #include "MagickCore/string-private.h"
 
 /*
@@ -200,7 +201,7 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
   int argc,char **argv,char **metadata,ExceptionInfo *exception)
 {
 #define CompareEpsilon  (1.0e-06)
-#define DefaultDissimilarityThreshold  0.31830988618379067154
+#define DefaultDissimilarityThreshold  (1.0/MagickPI)
 #define DefaultSimilarityThreshold  (-1.0)
 #define DestroyCompare() \
 { \
@@ -254,7 +255,8 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
   MagickBooleanType
     fire,
     pend,
-    respect_parenthesis,
+    respect_parentheses,
+    similar = MagickTrue,
     subimage_search;
 
   MagickStatusType
@@ -303,6 +305,7 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
   dissimilarity_threshold=DefaultDissimilarityThreshold;
   similarity_threshold=DefaultSimilarityThreshold;
   distortion=0.0;
+  similarity_metric=0.0;
   format=(char *) NULL;
   j=1;
   k=0;
@@ -311,7 +314,7 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
   option=(char *) NULL;
   pend=MagickFalse;
   reconstruct_image=NewImageList();
-  respect_parenthesis=MagickFalse;
+  respect_parentheses=MagickFalse;
   status=MagickTrue;
   subimage_search=MagickFalse;
   /*
@@ -960,7 +963,7 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
           }
         if (LocaleNCompare("respect-parentheses",option+1,17) == 0)
           {
-            respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
+            respect_parentheses=(*option == '-') ? MagickTrue : MagickFalse;
             break;
           }
         if (LocaleCompare("rotate",option+1) == 0)
@@ -1170,70 +1173,96 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
     {
       similarity_image=SimilarityImage(image,reconstruct_image,metric,
         similarity_threshold,&offset,&similarity_metric,exception);
-      if (similarity_metric > dissimilarity_threshold)
-        ThrowCompareException(ImageError,"ImagesTooDissimilar",image->filename);
+      if (dissimilarity_threshold == DefaultDissimilarityThreshold)
+        switch (metric)
+        {
+          case PhaseCorrelationErrorMetric:
+          case PeakSignalToNoiseRatioErrorMetric:
+          {
+            dissimilarity_threshold=1.0;
+            break;
+          }
+          default:
+            break;
+        }
+      if (similarity_metric > (dissimilarity_threshold+MagickEpsilon))
+        (void) ThrowMagickException(exception,GetMagickModule(),ImageWarning,
+          "ImagesTooDissimilar","`%s'",image->filename);
     }
-  if ((reconstruct_image->columns == image->columns) &&
-       (reconstruct_image->rows == image->rows))
+  if (similarity_image == (Image *) NULL)
     difference_image=CompareImages(image,reconstruct_image,metric,&distortion,
       exception);
   else
-    if (similarity_image == (Image *) NULL)
-      difference_image=CompareImages(image,reconstruct_image,metric,&distortion,
-        exception);
-    else
-      {
-        Image
-          *composite_image;
+    {
+      Image
+        *composite_image;
 
-        /*
-          Determine if reconstructed image is a subimage of the image.
-        */
-        composite_image=CloneImage(image,0,0,MagickTrue,exception);
-        if (composite_image == (Image *) NULL)
-          difference_image=CompareImages(image,reconstruct_image,metric,
+      /*
+        Determine if reconstructed image is a subimage of the image.
+      */
+      composite_image=CloneImage(image,0,0,MagickTrue,exception);
+      if (composite_image == (Image *) NULL)
+        difference_image=CompareImages(image,reconstruct_image,metric,
+          &distortion,exception);
+      else
+        {
+          Image
+            *distort_image;
+
+          RectangleInfo
+            page;
+
+          (void) CompositeImage(composite_image,reconstruct_image,
+            CopyCompositeOp,MagickTrue,offset.x,offset.y,exception);
+          difference_image=CompareImages(image,composite_image,metric,
             &distortion,exception);
-        else
-          {
-            Image
-              *distort_image;
+          if (difference_image != (Image *) NULL)
+            {
+              difference_image->page.x=offset.x;
+              difference_image->page.y=offset.y;
+            }
+          composite_image=DestroyImage(composite_image);
+          page.width=reconstruct_image->columns;
+          page.height=reconstruct_image->rows;
+          page.x=offset.x;
+          page.y=offset.y;
+          distort_image=CropImage(image,&page,exception);
+          if (distort_image != (Image *) NULL)
+            {
+              Image
+                *sans_image;
 
-            RectangleInfo
-              page;
-
-            (void) CompositeImage(composite_image,reconstruct_image,
-              CopyCompositeOp,MagickTrue,offset.x,offset.y,exception);
-            difference_image=CompareImages(image,composite_image,metric,
-              &distortion,exception);
-            if (difference_image != (Image *) NULL)
-              {
-                difference_image->page.x=offset.x;
-                difference_image->page.y=offset.y;
-              }
-            composite_image=DestroyImage(composite_image);
-            page.width=reconstruct_image->columns;
-            page.height=reconstruct_image->rows;
-            page.x=offset.x;
-            page.y=offset.y;
-            distort_image=CropImage(image,&page,exception);
-            if (distort_image != (Image *) NULL)
-              {
-                Image
-                  *sans_image;
-
-                sans_image=CompareImages(distort_image,reconstruct_image,metric,
-                  &distortion,exception);
-                distort_image=DestroyImage(distort_image);
-                if (sans_image != (Image *) NULL)
-                  sans_image=DestroyImage(sans_image);
-              }
-          }
-        if (difference_image != (Image *) NULL)
-          {
-            AppendImageToList(&difference_image,similarity_image);
-            similarity_image=(Image *) NULL;
-          }
-      }
+              sans_image=CompareImages(distort_image,reconstruct_image,metric,
+                &distortion,exception);
+              distort_image=DestroyImage(distort_image);
+              if (sans_image != (Image *) NULL)
+                sans_image=DestroyImage(sans_image);
+            }
+        }
+      if (difference_image != (Image *) NULL)
+        {
+          AppendImageToList(&difference_image,similarity_image);
+          similarity_image=(Image *) NULL;
+        }
+    }
+  switch (metric)
+  {
+    case DotProductCorrelationErrorMetric:
+    case NormalizedCrossCorrelationErrorMetric:
+    {
+      distortion=1.0-distortion;
+      similarity_metric=1.0-similarity_metric;
+      break;
+    }
+    case PhaseCorrelationErrorMetric:
+    {
+      distortion=1.0-distortion;
+      break;
+    }
+    default: break;
+  }
+  if (fabs(distortion) > CompareEpsilon)
+    similar=MagickFalse;
   if (difference_image == (Image *) NULL)
     status=0;
   else
@@ -1247,7 +1276,14 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
         {
           switch (metric)
           {
+            case AbsoluteErrorMetric:
+            case DotProductCorrelationErrorMetric:
             case FuzzErrorMetric:
+            case NormalizedCrossCorrelationErrorMetric:
+            case PerceptualHashErrorMetric:
+            case PhaseCorrelationErrorMetric:
+            case StructuralSimilarityErrorMetric:
+            case StructuralDissimilarityErrorMetric:
             case MeanAbsoluteErrorMetric:
             case MeanSquaredErrorMetric:
             case PeakAbsoluteErrorMetric:
@@ -1261,16 +1297,7 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
             case PeakSignalToNoiseRatioErrorMetric:
             {
               (void) FormatLocaleFile(stderr,"%.*g (%.*g)",GetMagickPrecision(),
-                distortion,GetMagickPrecision(),0.01*distortion);
-              break;
-            }
-            case AbsoluteErrorMetric:
-            case NormalizedCrossCorrelationErrorMetric:
-            case PerceptualHashErrorMetric:
-            case StructuralSimilarityErrorMetric:
-            case StructuralDissimilarityErrorMetric:
-            {
-              (void) FormatLocaleFile(stderr,"%.*g",GetMagickPrecision(),
+                (double) QuantumRange*distortion,GetMagickPrecision(),
                 distortion);
               break;
             }
@@ -1286,9 +1313,10 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
               break;
           }
           if (subimage_search != MagickFalse)
-            (void) FormatLocaleFile(stderr," @ %.20g,%.20g",
+            (void) FormatLocaleFile(stderr," @ %.20g,%.20g [%.*g]",
               (double) difference_image->page.x,
-              (double) difference_image->page.y);
+              (double) difference_image->page.y,GetMagickPrecision(),
+              similarity_metric);
         }
       else
         {
@@ -1387,9 +1415,11 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
               break;
             }
             case AbsoluteErrorMetric:
+            case DotProductCorrelationErrorMetric:
             case NormalizedCrossCorrelationErrorMetric:
             case PeakSignalToNoiseRatioErrorMetric:
             case PerceptualHashErrorMetric:
+            case PhaseCorrelationErrorMetric:
             case StructuralSimilarityErrorMetric:
             case StructuralDissimilarityErrorMetric:
             {
@@ -1459,8 +1489,12 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
           channel_distortion=(double *) RelinquishMagickMemory(
             channel_distortion);
           if (subimage_search != MagickFalse)
-            (void) FormatLocaleFile(stderr,"   Offset: %.20g,%.20g\n",(double)
-              difference_image->page.x,(double) difference_image->page.y);
+            {
+              (void) FormatLocaleFile(stderr,"   Offset: %.20g,%.20g\n",(double)
+                difference_image->page.x,(double) difference_image->page.y);
+              (void) FormatLocaleFile(stderr,"   Similarity metric: %*g\n",
+                GetMagickPrecision(),similarity_metric);
+            }
         }
       (void) ResetImagePage(difference_image,"0x0+0+0");
       if (difference_image->next != (Image *) NULL)
@@ -1483,15 +1517,7 @@ WandExport MagickBooleanType CompareImagesCommand(ImageInfo *image_info,
       difference_image=DestroyImageList(difference_image);
     }
   DestroyCompare();
-  if ((metric == NormalizedCrossCorrelationErrorMetric) ||
-      (metric == StructuralSimilarityErrorMetric) ||
-      (metric == UndefinedErrorMetric))
-    {
-      if (fabs(distortion-1.0) > CompareEpsilon)
-        (void) SetImageOption(image_info,"compare:dissimilar","true");
-    }
-  else
-    if (fabs(distortion) > CompareEpsilon)
-      (void) SetImageOption(image_info,"compare:dissimilar","true");
+  if (similar == MagickFalse)
+    (void) SetImageOption(image_info,"compare:dissimilar","true");
   return(status != 0 ? MagickTrue : MagickFalse);
 }
